/*
 * Copyright 2015 - 2024 gary@drinkingtea.net
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

#pragma once

#if __has_include(<source_location>)

#include <source_location>

#else

#include "types.hpp"

namespace std {

class source_location {
	private:
		struct __impl {
			char const*_M_file_name{};
			char const*_M_function_name{};
			uint32_t _M_line{};
			uint32_t _M_column{};
		};
		__impl const*mData = nullptr;
		using Raw = decltype(__builtin_source_location());

	public:
		constexpr source_location() noexcept = default;

		static consteval source_location current(Raw const pSl = __builtin_source_location()) noexcept {
			source_location sl;
			sl.mData = static_cast<__impl const*>(pSl);
			return sl;
		}

		constexpr uint32_t line() const noexcept {
			return mData != nullptr ? mData->_M_line : 0;
		}

		constexpr uint32_t column() const noexcept {
			return mData != nullptr ? mData->_M_column : 0;
		}

		constexpr ox::CString file_name() const noexcept {
			return mData != nullptr ? mData->_M_file_name : "";
		}

		constexpr ox::CString function_name() const noexcept {
			return mData != nullptr ? mData->_M_function_name : "";
		}
};

}

#endif
